#include <Trade\Trade.mqh>

CTrade trade; // Dichiarazione globale della variabile trade

// Variabili di input
input double PartialProfitTarget = 250.0; // Guadagno in valuta corrente per la chiusura parziale
input double PartialClosePercentage = 0.5; // Percentuale della posizione da chiudere parzialmente
input double FullProfitTarget = 300.0; // Guadagno in valuta corrente per la chiusura totale

void OnTick() {
   //+------------------------------------------------------------------+
   //|           Logica per eseguire parzialmente e breakeven acquisto 1|
   //+------------------------------------------------------------------+

   for (int i = 0; i < PositionsTotal(); i++) {
      ulong iTicket = PositionGetTicket(i);
      if (PositionGetInteger(POSITION_TYPE, iTicket) == POSITION_TYPE_BUY) {
         double PClose = PositionGetDouble(POSITION_PRICE_OPEN, iTicket) + PartialProfitTarget * _Point; // Target di chiusura parziale
         if (PositionGetDouble(POSITION_PRICE_CURRENT, iTicket) > PClose &&
             (PositionGetDouble(POSITION_SL, iTicket) < PositionGetDouble(POSITION_PRICE_OPEN, iTicket))) {
            double partialVolume = NormalizeDouble(PositionGetDouble(POSITION_VOLUME, iTicket) * PartialClosePercentage, 2); // Volume di chiusura parziale
            if (!trade.PositionClosePartial(iTicket, partialVolume, ULONG_MAX)) {
               Print("PositionClosePartial error ", trade.ResultRetcode());
            }
            else if (!trade.PositionModify(iTicket, PositionGetDouble(POSITION_PRICE_OPEN, iTicket), PositionGetDouble(POSITION_TP, iTicket))) { // Breakeven
               Print("PositionModify error ", trade.ResultRetcode());
            }
         }
      }
   }

   //+------------------------------------------------------------------+
   //|           Logica per eseguire parzialmente e breakeven acquisto 2|
   //+------------------------------------------------------------------+

   for (int i = 0; i < PositionsTotal(); i++) {
      ulong iTicket = PositionGetTicket(i);
      if (PositionGetInteger(POSITION_TYPE, iTicket) == POSITION_TYPE_BUY) {
         double PClose = PositionGetDouble(POSITION_PRICE_OPEN, iTicket) + FullProfitTarget * _Point; // Target di chiusura parziale
         if (PositionGetDouble(POSITION_PRICE_CURRENT, iTicket) > PClose &&
             (PositionGetDouble(POSITION_SL, iTicket) < PositionGetDouble(POSITION_PRICE_OPEN, iTicket))) {
            double partialVolume = NormalizeDouble(PositionGetDouble(POSITION_VOLUME, iTicket) * PartialClosePercentage, 2); // Volume di chiusura parziale
            if (!trade.PositionClosePartial(iTicket, partialVolume, ULONG_MAX)) {
               Print("PositionClosePartial error ", trade.ResultRetcode());
            }
            else if (!trade.PositionModify(iTicket, PositionGetDouble(POSITION_PRICE_OPEN, iTicket), PositionGetDouble(POSITION_TP, iTicket))) { // Breakeven
               Print("PositionModify error ", trade.ResultRetcode());
            }
         }
      }
   }

   //+------------------------------------------------------------------+
   //|            Logica per eseguire parzialmente e breakeven vendita 1|
   //+------------------------------------------------------------------+

   for (int i = 0; i < PositionsTotal(); i++) {
      ulong iTicket = PositionGetTicket(i);
      if (PositionGetInteger(POSITION_TYPE, iTicket) == POSITION_TYPE_SELL) {
         double PClose = PositionGetDouble(POSITION_PRICE_OPEN, iTicket) - PartialProfitTarget * _Point; // Target di chiusura parziale
         if (PositionGetDouble(POSITION_PRICE_CURRENT, iTicket) < PClose &&
             (PositionGetDouble(POSITION_SL, iTicket) > PositionGetDouble(POSITION_PRICE_OPEN, iTicket))) {
            double partialVolume = NormalizeDouble(PositionGetDouble(POSITION_VOLUME, iTicket) * PartialClosePercentage, 2); // Volume di chiusura parziale
            if (!trade.PositionClosePartial(iTicket, partialVolume, ULONG_MAX)) {
               Print("PositionClosePartial error ", trade.ResultRetcode());
            }
            else if (!trade.PositionModify(iTicket, PositionGetDouble(POSITION_PRICE_OPEN, iTicket), PositionGetDouble(POSITION_TP, iTicket))) { // Breakeven
               Print("PositionModify error ", trade.ResultRetcode());
            }
         }
      }
   }

   //+------------------------------------------------------------------+
   //|            Logica per eseguire parzialmente e breakeven vendita 2|
   //+------------------------------------------------------------------+

   for (int i = 0; i < PositionsTotal(); i++) {
      ulong iTicket = PositionGetTicket(i);
      if (PositionGetInteger(POSITION_TYPE, iTicket) == POSITION_TYPE_SELL) {
         double PClose = PositionGetDouble(POSITION_PRICE_OPEN, iTicket) - FullProfitTarget * _Point; // Target di chiusura parziale
         if (PositionGetDouble(POSITION_PRICE_CURRENT, iTicket) < PClose &&
             (PositionGetDouble(POSITION_SL, iTicket) > PositionGetDouble(POSITION_PRICE_OPEN, iTicket))) {
            double partialVolume = NormalizeDouble(PositionGetDouble(POSITION_VOLUME, iTicket) * PartialClosePercentage, 2); // Volume di chiusura parziale
            if (!trade.PositionClosePartial(iTicket, partialVolume, ULONG_MAX)) {
               Print("PositionClosePartial error ", trade.ResultRetcode());
            }
            else if (!trade.PositionModify(iTicket, PositionGetDouble(POSITION_PRICE_OPEN, iTicket), PositionGetDouble(POSITION_TP, iTicket))) { // Breakeven
               Print("PositionModify error ", trade.ResultRetcode());
            }
         }
      }
   }
}
